#!/usr/bin/env ruby
#####################################################################################
# Copyright 2016 Kenneth Evensen <kenneth.evensen@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
######################################################################################
#
# Contact Info: <kenneth.evensen@redhat.com> and <lester@redhat.com>
#
######################################################################################

require 'rest-client'
require 'json'
require 'singleton'

class OcpClient
  include Singleton

  attr_reader :url
  attr_reader :response

	def initialize
		@method = nil
		@url = nil
		@token = nil
		@noverifyssl = false
    @pretty = false
    @clientcert = nil
    @clientkey = nil
    @clientca = nil
    @clientcafile = nil
    @debug = nil
    @response = nil
	end

  def setup(url, noverifyssl, pretty, debug, token,
                clientcertfile, clientkeyfile, clientcafile)
    @noverifyssl = noverifyssl
    @pretty = pretty
    @debug = debug

    @url = url

    unless clientcertfile.nil? or clientkeyfile.nil?
      @clientcertfile = clientcertfile
      @clientkeyfile = clientkeyfile
    end

    unless clientcafile.nil?
      @clientcafile = clientcafile
    end

    unless @clientcertfile.nil? or @clientkeyfile.nil?
      @clientcert = OpenSSL::X509::Certificate.new File.read @clientcertfile
      @clientkey = OpenSSL::PKey::RSA.new File.read @clientkeyfile
    end

    unless @clientcafile.nil?
      @clientca = OpenSSL::X509::Certificate.new File.read @clientcafile
    end

    unless token.nil?
      @token = token
    end

    validate_options

  end

  def setup_by_config_file(configfile, pretty, debug)
    config = read_config(configfile)
    @debug=debug
    @pretty=pretty

    unless config.nil?
      @token = config["connection"]["token"]
      @url = config["connection"]["master"]
      @noverifyssl = config["connection"]["no_verify_ssl"]
      @clientcertfile = config["connection"]["client-certificate-file"]
      @clientkeyfile = config["connection"]["client-key-file"]
      @clientcafile = config["connection"]["client-ca-file"]
    end

    unless @clientcertfile.nil? or @clientkeyfile.nil?
      @clientcert = OpenSSL::X509::Certificate.new File.read @clientcertfile
      @clientkey = OpenSSL::PKey::RSA.new File.read @clientkeyfile
    end

    unless @clientcafile.nil?
      @clientca = OpenSSL::X509::Certificate.new File.read @clientcafile
    end

    validate_options
  end


  def get(location)
    @method = :get
	  return call(location,nil,:get)
  end

  def post(location, body)
    @method = :post
    return call(location,body,:post)
  end

  def put(location, body)
    @method = :put
    return call(location,body,:put)
  end

  def delete(location)
    @method = :delete
    return call(location,nil,:delete)
  end

  private

  def call(location, body, method)
    if !@clientcert.nil? and !@clientkey.nil?
      if @debug
        puts "Calling by Certificate"
      end
      return callbycert(location, body, method)
    else
      if @debug
        puts "Calling by Token"
      end
      return callbytoken(location, body, method)
    end
  end

  def callbytoken(location, body=nil, method)

    if @debug
      puts "Client Token - #{@token}"
      puts "URL - #{@url}#{location}"
      puts "Method - #{method}"
      unless body.nil?
        puts "Body  - #{JSON.pretty_generate(body)}"
      end
    end

    response = nil
    begin
      if body.nil?
        response = RestClient::Request.new(
            :method => method,
            :url => @url+location+"?"+query_params,
            :verify_ssl => !@noverifyssl,
            :headers => { :accept => :json,
            :content_type => :json,
            :Authorization => "Bearer #{@token}"}
        ).execute
      else
        response = RestClient::Request.new(
            :method => method,
            :url => @url+location+"?"+query_params,
            :verify_ssl => !@noverifyssl,
            :headers => { :accept => :json,
            :content_type => :json,
            :Authorization => "Bearer #{@token}"},
            :payload => JSON.generate(body)
        ).execute
      end
    rescue => exception
      if @debug and !exception.nil?
        puts "Exception: " << exception.to_s
      end
      @response = exception.to_s
    end

    if !response.nil? and @pretty
      results = JSON.pretty_generate(JSON.parse(response.to_str))
      @response = results
    elsif !response.nil?
      @response = response
    end

    return @response

  end

  def callbycert(location, body=nil, method)

    if @debug
      puts "Client Cert - #{@clientcert}"
      puts "Client Key - #{@clientkey}"
      puts "CA File - #{@clientcafile}"
      puts "URL - #{@url}#{location}"
      puts "Method - #{method}"
      unless body.nil?
        puts "Body  - #{JSON.pretty_generate(body)}"
      end
    end

    response = nil
    begin
      if body.nil?
        response = RestClient::Request.new(
            :method => method,
            :url => @url+location+"?"+query_params,
            :verify_ssl => !@noverifyssl,
            :ssl_client_cert => @clientcert,
            :ssl_client_key => @clientkey,
            :ssl_ca_file => @clientcafile,
            :headers => { :accept => :json,
            :content_type => :json}
        ).execute
      else
        response = RestClient::Request.new(
            :method => method,
            :url => @url+location+"?"+query_params,
            :verify_ssl => !@noverifyssl,
            :ssl_client_cert => @clientcert,
            :ssl_client_key => @clientkey,
            :ssl_ca_file => @clientcafile,
            :headers => { :accept => :json,
            :content_type => :json},
            :payload => JSON.generate(body)
        ).execute
      end
    rescue => exception
      if @debug and !exception.nil?
        puts "Exception: " << exception.to_s
      end
      @response = exception.to_s
    end

    if !response.nil? and @pretty
      results = JSON.pretty_generate(JSON.parse(response.to_str))
      @response = results
    elsif !response.nil?
      @response = response
    end

    return @response

  end

  def query_params
    return "pretty=#{@pretty}"
  end

  def validate_options

    if @token.nil? and (@clientcertfile.nil? or @clientkeyfile.nil?)
      raise MissingRequiredOptionException.new("token OR clientcertfile AND clientkeyfile")
    elsif @url.nil?
      raise MissingRequiredOptionException.new("master")
    end
  end

  def read_config(configfile)
      begin
        unless configfile.nil?
          return YAML.load_file(configfile)
        else
          return YAML.load_file("config.yaml")
        end
      rescue => exception
        puts exception.message
        puts "Unable to read config file #{configfile}.  Proceeding"
      end
  end

end
